/*=============================================================================
    Copyright (c) 2002-2003 Hartmut Kaiser
    http://spirit.sourceforge.net/

    Use, modification and distribution is subject to the Boost Software
    License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
    http://www.boost.org/LICENSE_1_0.txt)
=============================================================================*/
#ifndef FOO_REGEX_IPP
#define FOO_REGEX_IPP

///////////////////////////////////////////////////////////////////////////////
#include <boost/spirit/home/classic/core/primitives/impl/primitives.ipp>
#include <boost/spirit/home/classic/iterator/position_iterator_fwd.hpp>

///////////////////////////////////////////////////////////////////////////////
namespace foo {

namespace impl {

///////////////////////////////////////////////////////////////////////////////
// Some bits and pieces to help with position_iterator

template <typename Iterator>
struct position_iterator_base {
    typedef Iterator base;
};

template <typename ForwardIteratorT, typename PositionT, typename SelfT>
struct position_iterator_base<boost::spirit::classic::position_iterator<ForwardIteratorT, PositionT, SelfT> > {
    typedef typename boost::spirit::classic::position_iterator<ForwardIteratorT, PositionT, SelfT>::base_type type;
};

template <typename ForwardIteratorT, typename PositionT>
struct position_iterator_base<boost::spirit::classic::position_iterator2<ForwardIteratorT, PositionT> > {
    typedef typename boost::spirit::classic::position_iterator<ForwardIteratorT, PositionT>::base_type type;
};


template <typename Iterator>
inline Iterator get_position_iterator_base(Iterator x)
{
    return x;
};

template <typename ForwardIteratorT, typename PositionT, typename SelfT>
inline ForwardIteratorT get_position_iterator_base(
        boost::spirit::classic::position_iterator<ForwardIteratorT, PositionT, SelfT> x)
{
    return x.base();
}

template <typename ForwardIteratorT, typename PositionT>
inline ForwardIteratorT get_position_iterator_base(
        boost::spirit::classic::position_iterator2<ForwardIteratorT, PositionT> x)
{
    return x.base();
}

template <typename Iterator>
inline Iterator move_iterator_to(Iterator it, Iterator pos) {
    return pos;
}

template <typename Iterator1, typename Iterator2>
inline Iterator1 move_iterator_to(Iterator1 it, Iterator2 pos) {
    while(it.base() != pos) ++it;
    return it;
}

///////////////////////////////////////////////////////////////////////////////
//
inline const char* rx_prefix(char) { return "\\A"; }
inline const wchar_t* rx_prefix(wchar_t) { return L"\\A"; }

///////////////////////////////////////////////////////////////////////////////
//
//  rx_parser class
//
///////////////////////////////////////////////////////////////////////////////
template <typename CharT = char>
class rx_parser : public boost::spirit::classic::parser<rx_parser<CharT> > {

public:
    typedef std::basic_string<CharT> string_t;
    typedef rx_parser<CharT> self_t;

    rx_parser(CharT const *first, CharT const *last)
    { 
        rxstr = string_t(rx_prefix(CharT())) + string_t(first, last); 
    }

    rx_parser(CharT const *first)
    { 
        rxstr = string_t(rx_prefix(CharT())) + 
            string_t(first, boost::spirit::classic::impl::get_last(first)); 
    }

    template <typename ScannerT>
    typename boost::spirit::classic::parser_result<self_t, ScannerT>::type
    parse(ScannerT const& scan) const
    {
        boost::match_results<typename position_iterator_base<
            typename ScannerT::iterator_t>::type> what;
        boost::regex_search(
            get_position_iterator_base(scan.first), 
            get_position_iterator_base(scan.last),
            what, rxstr, boost::match_default);
            
        if (!what[0].matched)
            return scan.no_match();

        typename ScannerT::iterator_t it = scan.first;

        scan.first = move_iterator_to(it, what[0].second);
        return scan.create_match(what[0].length(), boost::spirit::classic::nil_t(),
            move_iterator_to(it, what[0].first), scan.first);
    }

private:
#if BOOST_VERSION >= 013300
    boost::basic_regex<CharT> rxstr;       // regular expression to match
#else
    boost::reg_expression<CharT> rxstr;    // regular expression to match
#endif
};

}   // namespace impl

}   // namespace foo

#endif // FOO_REGEX_IPP
